@@ -6,16 +6,21 @@ import re
 import subprocess
 import tempfile
 from pathlib import Path
-from typing import Callable, Iterable
+from typing import Callable, Iterable, Sequence
 
 import astor
 from google.protobuf.descriptor_pb2 import FileDescriptorSet
 
-from .rewrite import ImportRewriter, build_import_rewrite, register_import_rewrite
+from .rewrite import ImportRewriter, build_import_rewrite
 
 _PROTO_SUFFIX_PATTERN = re.compile(r"^(.+)\.proto$")
 
 
+def _remove_proto_suffix(name: str) -> str:
+    """Remove the `.proto` suffix from `name`."""
+    return _PROTO_SUFFIX_PATTERN.sub(r"\1", name)
+
+
 class FileDescriptorSetGenerator(abc.ABC):
     def __init__(self, fdset_generator_binary: str) -> None:
         self.fdset_generator_binary = fdset_generator_binary
@@ -26,36 +31,46 @@ class FileDescriptorSetGenerator(abc.ABC):
 
     def fix_imports(
         self,
-        generated_python_dir: Path,
-        create_init: bool,
+        python_out: Path,
+        create_package: bool,
         overwrite_callback: Callable[[Path, str], None],
+        module_suffixes: Sequence[str],
     ) -> None:
+        rewriters = {}
         fdset = FileDescriptorSet.FromString(self.generate_file_descriptor_set_bytes())
 
         for fd in fdset.file:
-            fd_name = _PROTO_SUFFIX_PATTERN.sub(r"\1", fd.name)
+            fd_name = _remove_proto_suffix(fd.name)
+            rewriters[fd_name] = rewriter = ImportRewriter()
+            # services live outside of the corresponding generated Python
+            # module, but they import it so we register a rewrite for the
+            # current proto as a dependency of itself to handle the case
+            # of services
+            rewriter.register_import_rewrite(build_import_rewrite(fd_name, fd_name))
             for dep in fd.dependency:
-                register_import_rewrite(
-                    build_import_rewrite(fd_name, _PROTO_SUFFIX_PATTERN.sub(r"\1", dep))
+                dep_name = _remove_proto_suffix(dep)
+                rewriter.register_import_rewrite(
+                    build_import_rewrite(fd_name, dep_name)
                 )
 
-        rewriter = ImportRewriter()
-
         # only rewrite things with dependencies
-        for fd_name in (fd.name for fd in fdset.file if fd.dependency):
-            name = _PROTO_SUFFIX_PATTERN.sub(r"\1_pb2.py", fd_name)
-            python_file = generated_python_dir.joinpath(name)
-            raw_code = python_file.read_text()
-            module = ast.parse(raw_code)
-            new_module = rewriter.visit(module)
-            new_code = astor.to_source(new_module)
-            overwrite_callback(python_file, new_code)
+        for fd in fdset.file:
+            fd_name = _remove_proto_suffix(fd.name)
+            for suffix in module_suffixes:
+                py_name = f"{fd_name}{suffix}"
+                python_file = python_out.joinpath(py_name)
+                if python_file.exists():
+                    raw_code = python_file.read_text()
+                    module = ast.parse(raw_code)
+                    new_module = rewriters[fd_name].visit(module)
+                    new_code = astor.to_source(new_module)
+                    overwrite_callback(python_file, new_code)
 
-        if create_init:
-            generated_python_dir.joinpath("__init__.py").touch(exist_ok=True)
+        if create_package:
+            python_out.joinpath("__init__.py").touch(exist_ok=True)
 
             # recursively create packages
-            for dir_entry in generated_python_dir.rglob("*"):
+            for dir_entry in python_out.rglob("*"):
                 if dir_entry.is_dir():
                     dir_entry.joinpath("__init__.py").touch(exist_ok=True)
 
