@@ -3,9 +3,10 @@ from __future__ import annotations
 import ast
 import collections
 import collections.abc
+from ast import AST
 from typing import Any, Callable, NamedTuple, Sequence, Union
 
-Node = Union[ast.AST, Sequence[ast.AST]]
+Node = Union[AST, Sequence[AST]]
 
 
 class Replacement(NamedTuple):
@@ -43,14 +44,12 @@ def matches(value: Node, pattern: Node) -> bool:
         return all(map(matches, value, pattern))
 
     # primitive value, such as None, True, False, etc.
-    if not isinstance(value, ast.AST) and not isinstance(pattern, ast.AST):
+    if not isinstance(value, AST) and not isinstance(pattern, AST):
         return value == pattern
 
+    assert isinstance(value, AST), f"value is not an AST node: {type(value).__name__}"
     assert isinstance(
-        value, ast.AST
-    ), f"value is not an AST node: {type(value).__name__}"
-    assert isinstance(
-        pattern, ast.AST
+        pattern, AST
     ), f"pattern is not an AST node: {type(pattern).__name__}"
 
     fields = [
@@ -68,27 +67,24 @@ class Rewriter:
     """AST pattern matching to enable rewrite rules."""
 
     def __init__(self) -> None:
-        self.funcs: list[tuple[ast.AST, Callable[[ast.AST], ast.AST]]] = []
+        self.funcs: list[tuple[AST, Callable[[AST], AST]]] = []
 
     def register(
-        self, pattern: ast.AST
-    ) -> Callable[[Callable[[ast.AST], ast.AST]], Callable[[ast.AST], ast.AST]]:
-        def wrapper(f: Callable[[ast.AST], ast.AST]) -> Callable[[ast.AST], ast.AST]:
+        self, pattern: AST
+    ) -> Callable[[Callable[[AST], AST]], Callable[[AST], AST]]:
+        def wrapper(f: Callable[[AST], AST]) -> Callable[[AST], AST]:
             self.funcs.append((pattern, f))
             return f
 
         return wrapper
 
-    def __call__(self, node: ast.AST) -> ast.AST:
+    def __call__(self, node: AST) -> AST:
         for pattern, func in self.funcs:
             if matches(node, pattern):
                 return func(node)
         return node
 
 
-rewrite = Rewriter()
-
-
 def build_import_rewrite(proto: str, dep: str) -> Replacement:
     """Construct a replacement import for `dep`.
 
@@ -110,12 +106,13 @@ def build_import_rewrite(proto: str, dep: str) -> Replacement:
     # compute the number of dots needed to get to the package root
     num_leading_dots = proto.count("/") + 1
     leading_dots = "." * num_leading_dots
-    if not import_parts:
-        old = f"import {part}_pb2 as {part}__pb2"
-        new = f"from {leading_dots} import {part}_pb2 as {part}__pb2"
-    else:
-        last_part = "__".join(f"{part}_pb2".split("_"))
+    last_part = "__".join(f"{part}_pb2".split("_"))
 
+    if not import_parts:
+        # underscores are doubled by protoc/buf
+        old = f"import {part}_pb2 as {last_part}"
+        new = f"from {leading_dots} import {part}_pb2 as {last_part}"
+    else:
         from_ = ".".join(import_parts)
         as_ = f"{'_dot_'.join(import_parts)}_dot_{last_part}"
 
@@ -125,22 +122,27 @@ def build_import_rewrite(proto: str, dep: str) -> Replacement:
     return Replacement(old=old, new=new)
 
 
-def register_import_rewrite(replacement: Replacement) -> None:
-    """Register a rewrite rule for turning `old` into `new`."""
-    (old_import,) = ast.parse(replacement.old).body
-    (new_import,) = ast.parse(replacement.new).body
-
-    def _rewrite(_: ast.AST, new_import: ast.AST = new_import) -> ast.AST:
-        return new_import
-
-    rewrite.register(old_import)(_rewrite)
-
-
 class ImportRewriter(ast.NodeTransformer):
     """A NodeTransformer to apply rewrite rules."""
 
-    def visit_Import(self, node: ast.Import) -> ast.AST:
-        return rewrite(node)
+    def __init__(self) -> None:
+        self.rewrite = Rewriter()
 
-    def visit_ImportFrom(self, node: ast.ImportFrom) -> ast.AST:
-        return rewrite(node)
+    def register_import_rewrite(self, replacement: Replacement) -> None:
+        """Register a rewrite rule for turning `old` into `new`."""
+        (old_import,) = ast.parse(replacement.old).body
+        (new_import,) = ast.parse(replacement.new).body
+
+        def _rewrite(_: AST, new_import: AST = new_import) -> AST:
+            return new_import
+
+        if all(not matches(old_import, pat) for pat, _ in self.rewrite.funcs):
+            self.rewrite.register(old_import)(_rewrite)
+
+        assert sum(matches(old_import, pat) for pat, _ in self.rewrite.funcs) == 1
+
+    def visit_Import(self, node: ast.Import) -> AST:
+        return self.rewrite(node)
+
+    def visit_ImportFrom(self, node: ast.ImportFrom) -> AST:
+        return self.rewrite(node)
