@@ -1,5 +1,4 @@
 import importlib
-import json
 import subprocess
 import sys
 from pathlib import Path
@@ -49,6 +48,7 @@ def check_proto_out(out: Path) -> None:
             "foo/bar/bizz_buzz",
             "from ..foo.bar import bizz_buzz_pb2 as foo_dot_bar_dot_bizz__buzz__pb2",
         ),
+        ("a/b", "a/b", "from ..a import b_pb2 as a_dot_b__pb2"),
     ],
 )
 def test_build_import_rewrite(proto: str, dep: str, expected: str) -> None:
@@ -56,61 +56,6 @@ def test_build_import_rewrite(proto: str, dep: str, expected: str) -> None:
     assert new == expected
 
 
-@pytest.fixture
-def cli() -> CliRunner:
-    return CliRunner()
-
-
-@pytest.fixture
-def this_proto(tmp_path: Path) -> Path:
-    code = """
-syntax = "proto3";
-
-import "other.proto";
-import "baz/bizz_buzz.proto";
-
-package protoletariat.test;
-
-message Test {
-    Other test = 1;
-    protoletariat.test.baz.BuzzBuzz baz = 2;
-}
-"""
-    p = tmp_path.joinpath("this.proto")
-    p.write_text(code)
-    return p
-
-
-@pytest.fixture
-def other_proto(tmp_path: Path) -> Path:
-    code = """
-syntax = "proto3";
-
-package protoletariat.test;
-
-message Other {}
-"""
-    p = tmp_path.joinpath("other.proto")
-    p.write_text(code)
-    return p
-
-
-@pytest.fixture
-def baz_bizz_buzz_other_proto(tmp_path: Path) -> Path:
-    code = """
-syntax = "proto3";
-
-package protoletariat.test.baz;
-
-message BuzzBuzz {}
-"""
-    baz = tmp_path.joinpath("baz")
-    baz.mkdir()
-    p = baz.joinpath("bizz_buzz.proto")
-    p.write_text(code)
-    return p
-
-
 def test_cli(
     cli: CliRunner,
     tmp_path: Path,
@@ -129,11 +74,7 @@ def test_cli(
             str(other_proto),
             str(baz_bizz_buzz_other_proto),
             "--proto_path",
-            str(this_proto.parent),
-            "--proto_path",
-            str(other_proto.parent),
-            "--proto_path",
-            str(baz_bizz_buzz_other_proto),
+            str(tmp_path),
             "--python_out",
             str(out),
         ]
@@ -144,20 +85,17 @@ def test_cli(
     result = cli.invoke(
         main,
         [
-            "-g",
+            "-o",
             str(out),
-            "--no-overwrite",
+            "--not-in-place",
             "protoc",
             "-p",
-            str(this_proto.parent),
-            "-p",
-            str(other_proto.parent),
-            "-p",
-            str(baz_bizz_buzz_other_proto.parent),
+            str(tmp_path),
             str(this_proto),
             str(other_proto),
             str(baz_bizz_buzz_other_proto),
         ],
+        catch_exceptions=False,
     )
     assert result.exit_code == 0
 
@@ -171,21 +109,18 @@ def test_cli(
     result = cli.invoke(
         main,
         [
-            "-g",
+            "-o",
             str(out),
-            "--overwrite",
-            "--create-init",
+            "--in-place",
+            "--create-package",
             "protoc",
             "-p",
-            str(this_proto.parent),
-            "-p",
-            str(other_proto.parent),
-            "-p",
-            str(baz_bizz_buzz_other_proto.parent),
+            str(tmp_path),
             str(this_proto),
             str(other_proto),
             str(baz_bizz_buzz_other_proto),
         ],
+        catch_exceptions=False,
     )
     assert result.exit_code == 0
 
@@ -198,90 +133,6 @@ def test_cli(
     assert str(tmp_path) not in sys.path
 
 
-@pytest.fixture
-def thing1_text() -> str:
-    return """
-// thing1.proto
-syntax = "proto3";
-
-import "thing2.proto";
-
-package things;
-
-message Thing1 {
-  Thing2 thing2 = 1;
-}
-"""
-
-
-@pytest.fixture
-def thing1(thing1_text: str, tmp_path: Path) -> Path:
-    p = tmp_path.joinpath("thing1.proto")
-    p.write_text(thing1_text)
-    return p
-
-
-@pytest.fixture
-def thing2_text() -> str:
-    return """
-// thing2.proto
-syntax = "proto3";
-
-package things;
-
-message Thing2 {
-  string data = 1;
-}
-"""
-
-
-@pytest.fixture
-def thing2(thing2_text: str, tmp_path: Path) -> Path:
-    code = """
-// thing2.proto
-syntax = "proto3";
-
-package things;
-
-message Thing2 {
-  string data = 1;
-}
-"""
-    p = tmp_path.joinpath("thing2.proto")
-    p.write_text(code)
-    return p
-
-
-@pytest.fixture
-def thing1_nested_text() -> str:
-    return """
-// thing1.proto
-syntax = "proto3";
-
-import "a/b/c/thing2.proto";
-
-package thing1.a;
-
-message Thing1 {
-  thing2.a.b.c.Thing2 thing2 = 1;
-}
-"""
-
-
-@pytest.fixture
-def thing2_nested_text() -> str:
-    return """
-// thing2.proto
-syntax = "proto3";
-
-package thing2.a.b.c;
-
-message Thing2 {
-  string data = 1;
-}
-"""
-
-
 def test_example_protoc(
     cli: CliRunner,
     tmp_path: Path,
@@ -296,21 +147,19 @@ def test_example_protoc(
             str(thing1),
             str(thing2),
             "--proto_path",
-            str(thing1.parent),
-            "--proto_path",
-            str(thing2.parent),
+            str(tmp_path),
             "--python_out",
             str(out),
-        ]
+        ],
     )
 
     result = cli.invoke(
         main,
         [
-            "-g",
+            "-o",
             str(out),
-            "--no-overwrite",
-            "--create-init",
+            "--not-in-place",
+            "--create-package",
             "protoc",
             "-p",
             str(thing1.parent),
@@ -319,6 +168,7 @@ def test_example_protoc(
             str(thing1),
             str(thing2),
         ],
+        catch_exceptions=False,
     )
     assert result.exit_code == 0
 
@@ -333,117 +183,11 @@ def test_example_buf(
     tmp_path: Path,
     thing1: Path,
     thing2: Path,
+    buf_yaml: Path,
+    buf_gen_yaml: Path,
+    out: Path,
     monkeypatch: pytest.MonkeyPatch,
 ) -> None:
-    out = tmp_path.joinpath("out")
-    out.mkdir()
-
-    monkeypatch.chdir(tmp_path)
-    with tmp_path.joinpath("buf.yaml").open(mode="w") as f:
-        json.dump(
-            {
-                "version": "v1",
-                "lint": {"use": ["DEFAULT"]},
-                "breaking": {"use": ["FILE"]},
-            },
-            f,
-        )
-
-    with tmp_path.joinpath("buf.gen.yaml").open(mode="w") as f:
-        json.dump(
-            {
-                "version": "v1",
-                "plugins": [
-                    {
-                        "name": "python",
-                        "out": str(out),
-                    }
-                ],
-            },
-            f,
-        )
-
-    subprocess.check_call(
-        [
-            "protoc",
-            str(thing1),
-            str(thing2),
-            "--proto_path",
-            str(thing1.parent),
-            "--proto_path",
-            str(thing2.parent),
-            "--python_out",
-            str(out),
-        ],
-    )
-
-    check_proto_out(out)
-
-    result = cli.invoke(
-        main,
-        [
-            "-g",
-            str(out),
-            "--no-overwrite",
-            "--create-init",
-            "buf",
-        ],
-    )
-    assert result.exit_code == 0
-
-    expected_lines = ["from . import thing2_pb2 as thing2__pb2"]
-    check_import_lines(result, expected_lines)
-
-    assert out.joinpath("__init__.py").exists()
-
-
-def test_nested_buf(
-    cli: CliRunner,
-    tmp_path: Path,
-    thing1_nested_text: str,
-    thing2_nested_text: str,
-    monkeypatch: pytest.MonkeyPatch,
-) -> None:
-    out = tmp_path.joinpath("out_nested")
-    out.mkdir()
-
-    c = tmp_path / "a" / "b" / "c"
-    c.mkdir(parents=True, exist_ok=True)
-
-    thing2 = c / "thing2.proto"
-    thing2.write_text(thing2_nested_text)
-
-    d = tmp_path / "d"
-    d.mkdir()
-
-    thing1 = d / "thing1.proto"
-    thing1.write_text(thing1_nested_text)
-
-    monkeypatch.chdir(tmp_path)
-    with tmp_path.joinpath("buf.yaml").open(mode="w") as f:
-        json.dump(
-            {
-                "version": "v1",
-                "lint": {"use": ["DEFAULT"]},
-                "breaking": {"use": ["FILE"]},
-            },
-            f,
-        )
-
-    with tmp_path.joinpath("buf.gen.yaml").open(mode="w") as f:
-        json.dump(
-            {
-                "version": "v1",
-                "plugins": [
-                    {
-                        "name": "python",
-                        "out": str(out),
-                    },
-                ],
-            },
-            f,
-        )
-
     subprocess.check_call(
         [
             "protoc",
@@ -458,15 +202,75 @@ def test_nested_buf(
 
     check_proto_out(out)
 
+    monkeypatch.chdir(tmp_path)
+
     result = cli.invoke(
         main,
         [
-            "-g",
+            "-o",
             str(out),
-            "--overwrite",
-            "--create-init",
+            "--not-in-place",
+            "--create-package",
             "buf",
         ],
+        catch_exceptions=False,
+    )
+    assert result.exit_code == 0
+
+    expected_lines = ["from . import thing2_pb2 as thing2__pb2"]
+    check_import_lines(result, expected_lines)
+
+    assert out.joinpath("__init__.py").exists()
+
+
+def test_nested_buf(
+    cli: CliRunner,
+    tmp_path: Path,
+    buf_yaml: Path,
+    buf_gen_yaml_nested: Path,
+    out_nested: Path,
+    thing1_nested_text: str,
+    thing2_nested_text: str,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
+    c = tmp_path / "a" / "b" / "c"
+    c.mkdir(parents=True, exist_ok=True)
+
+    thing2 = c / "thing2.proto"
+    thing2.write_text(thing2_nested_text)
+
+    d = tmp_path / "d"
+    d.mkdir()
+
+    thing1 = d / "thing1.proto"
+    thing1.write_text(thing1_nested_text)
+
+    subprocess.check_call(
+        [
+            "protoc",
+            str(thing1),
+            str(thing2),
+            "--proto_path",
+            str(tmp_path),
+            "--python_out",
+            str(out_nested),
+        ],
+    )
+
+    check_proto_out(out_nested)
+
+    monkeypatch.chdir(tmp_path)
+
+    result = cli.invoke(
+        main,
+        [
+            "-o",
+            str(out_nested),
+            "--in-place",
+            "--create-package",
+            "buf",
+        ],
+        catch_exceptions=False,
     )
     assert result.exit_code == 0
 
@@ -479,8 +283,108 @@ def test_nested_buf(
         importlib.import_module("out_nested.d.thing1_pb2")
 
     # check that we created packages in the correct locations
-    assert out.joinpath("__init__.py").exists()
-    assert out.joinpath("a", "__init__.py").exists()
-    assert out.joinpath("a", "b", "__init__.py").exists()
-    assert out.joinpath("a", "b", "c", "__init__.py").exists()
-    assert out.joinpath("d", "__init__.py").exists()
+    assert out_nested.joinpath("__init__.py").exists()
+    assert out_nested.joinpath("a", "__init__.py").exists()
+    assert out_nested.joinpath("a", "b", "__init__.py").exists()
+    assert out_nested.joinpath("a", "b", "c", "__init__.py").exists()
+    assert out_nested.joinpath("d", "__init__.py").exists()
+
+
+@pytest.mark.xfail(
+    condition=sys.version_info[:2] == (3, 10),
+    raises=subprocess.CalledProcessError,
+    reason="grpc-cpp cannot be installed with conda using Python 3.10",
+)
+def test_grpc_buf(
+    cli: CliRunner,
+    tmp_path: Path,
+    thing1: Path,
+    thing2: Path,
+    buf_yaml: Path,
+    buf_gen_yaml_grpc: Path,
+    thing_service: Path,
+    out_grpc: Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
+    monkeypatch.chdir(tmp_path)
+    subprocess.check_call(["buf", "generate"])
+
+    check_proto_out(out_grpc)
+
+    result = cli.invoke(
+        main,
+        [
+            "-o",
+            str(out_grpc),
+            "--in-place",
+            "--create-package",
+            "buf",
+        ],
+        catch_exceptions=False,
+    )
+    assert result.exit_code == 0
+
+    service_module = out_grpc.joinpath("thing_service_pb2_grpc.py")
+    assert service_module.exists()
+
+    lines = service_module.read_text().splitlines()
+
+    assert "from . import thing1_pb2 as thing1__pb2" in lines
+    assert "import thing1_pb2 as thing1__pb2" not in lines
+
+    assert "from . import thing2_pb2 as thing2__pb2" in lines
+    assert "import thing2_pb2 as thing2__pb2" not in lines
+
+    assert out_grpc.joinpath("__init__.py").exists()
+
+    # check that we can import the thing
+    with monkeypatch.context() as m:
+        m.syspath_prepend(str(tmp_path))
+
+        importlib.import_module("out_grpc.thing1_pb2")
+        importlib.import_module("out_grpc.thing2_pb2")
+        importlib.import_module("out_grpc.thing_service_pb2_grpc")
+
+
+@pytest.mark.xfail(
+    condition=sys.version_info[:2] == (3, 10),
+    raises=subprocess.CalledProcessError,
+    reason="grpc-cpp cannot be installed with conda using Python 3.10",
+)
+def test_grpc_buf_no_imports(
+    cli: CliRunner,
+    tmp_path: Path,
+    buf_yaml: Path,
+    buf_gen_yaml_grpc_no_imports: Path,
+    no_imports_service: Path,
+    out_grpc_no_imports: Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
+    monkeypatch.chdir(tmp_path)
+    subprocess.check_call(["buf", "generate"])
+
+    check_proto_out(out_grpc_no_imports)
+
+    result = cli.invoke(
+        main,
+        [
+            "-o",
+            str(out_grpc_no_imports),
+            "--in-place",
+            "--create-package",
+            "buf",
+        ],
+        catch_exceptions=False,
+    )
+    assert result.exit_code == 0
+
+    service_module = out_grpc_no_imports.joinpath("no_imports_service_pb2_grpc.py")
+    assert service_module.exists()
+
+    assert out_grpc_no_imports.joinpath("__init__.py").exists()
+
+    # check that we can import the thing
+    with monkeypatch.context() as m:
+        m.syspath_prepend(str(tmp_path))
+
+        importlib.import_module("out_grpc_no_imports.no_imports_service_pb2_grpc")
